"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SwapForm = void 0;
const react_1 = require("@chakra-ui/react");
const yup_1 = require("@hookform/resolvers/yup");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const spl_token_bonding_1 = require("@strata-foundation/spl-token-bonding");
const react_2 = __importStar(require("react"));
const react_hook_form_1 = require("react-hook-form");
const ri_1 = require("react-icons/ri");
const yup = __importStar(require("yup"));
const hooks_1 = require("../../hooks");
const validationSchema = yup
    .object({
    topAmount: yup.number().required().moreThan(0),
    bottomAmount: yup.number().required().moreThan(0),
    slippage: yup.number().required().moreThan(0),
})
    .required();
const humanReadablePercentage = (u32) => {
    if (u32 && u32 !== 0) {
        return ((u32 / 4294967295) * 100).toFixed(2);
    }
    return 0;
};
function roundToDecimals(num, decimals) {
    return Math.trunc(num * Math.pow(10, decimals)) / Math.pow(10, decimals);
}
const SwapForm = ({ action, isSubmitting, onConnectWallet, onFlipTokens, onBuyBase, onSubmit, tokenBonding, curve, base, target, ownedBase, spendCap, feeAmount, }) => {
    const formRef = (0, react_2.useRef)();
    const { connected } = (0, wallet_adapter_react_1.useWallet)();
    const { awaitingApproval } = (0, hooks_1.useProvider)();
    const ftxPayLink = (0, hooks_1.useFtxPayLink)();
    const [rate, setRate] = (0, react_2.useState)("--");
    const [fee, setFee] = (0, react_2.useState)("--");
    const { register, handleSubmit, watch, reset, setValue, formState: { errors }, } = (0, react_hook_form_1.useForm)({
        defaultValues: {
            topAmount: undefined,
            bottomAmount: undefined,
            slippage: 1,
        },
        resolver: (0, yup_1.yupResolver)(validationSchema),
    });
    const isBaseSol = base.publicKey.equals(spl_token_bonding_1.SplTokenBonding.WRAPPED_SOL_MINT);
    const isBuying = action === "buy";
    const topAmount = watch("topAmount");
    const slippage = watch("slippage");
    const hasBaseAmount = ownedBase >= +(topAmount || 0);
    const moreThanSpendCap = +(topAmount || 0) > spendCap;
    const handleConnectWallet = () => onConnectWallet();
    const handleUseMax = () => setValue("topAmount", ownedBase >= spendCap ? spendCap : ownedBase);
    const handleFlipTokens = () => {
        onFlipTokens(tokenBonding.publicKey, isBuying ? "sell" : "buy");
        reset();
    };
    const handleBuyBase = () => {
        if (isBaseSol) {
            window.open(ftxPayLink);
        }
        else {
            onBuyBase(tokenBonding.publicKey);
        }
    };
    const handleSwap = (values) => __awaiter(void 0, void 0, void 0, function* () {
        yield onSubmit(values);
        reset();
    });
    (0, react_2.useEffect)(() => {
        if (topAmount && topAmount >= 0 && tokenBonding && curve) {
            if (isBuying) {
                const buyMax = curve.buyWithBaseAmount(+topAmount, tokenBonding.buyBaseRoyaltyPercentage, tokenBonding.buyTargetRoyaltyPercentage);
                setValue("bottomAmount", roundToDecimals(buyMax, 9));
                setRate(`${roundToDecimals(buyMax / topAmount, 9)}`);
            }
            else {
                const sellMax = curve.sellTargetAmount(+topAmount, tokenBonding.sellBaseRoyaltyPercentage, tokenBonding.sellTargetRoyaltyPercentage);
                setValue("bottomAmount", roundToDecimals(sellMax, 9));
                setRate(`${roundToDecimals(sellMax / topAmount, 9)}`);
            }
            setFee(`${feeAmount}`);
        }
        else {
            reset({ slippage: slippage });
            setRate("--");
            setFee("--");
        }
    }, [topAmount, feeAmount, setValue, setRate, tokenBonding, curve, slippage]);
    return (react_2.default.createElement(react_1.Box, { ref: formRef, w: "full" },
        react_2.default.createElement("form", { onSubmit: handleSubmit(handleSwap) },
            react_2.default.createElement(react_1.VStack, { spacing: 4, padding: 4, align: "stretch", color: "gray.500" },
                react_2.default.createElement(react_1.Flex, { flexDir: "column" },
                    react_2.default.createElement(react_1.Flex, { justifyContent: "space-between" },
                        react_2.default.createElement(react_1.Text, { color: "gray.600", fontSize: "xs" }, "You Pay"),
                        react_2.default.createElement(react_1.Link, { color: "indigo.500", fontSize: "xs", onClick: handleBuyBase },
                            "Buy More ",
                            base.ticker)),
                    react_2.default.createElement(react_1.InputGroup, { size: "lg" },
                        react_2.default.createElement(react_1.Input, Object.assign({ isInvalid: !!errors.topAmount, isDisabled: !connected, id: "topAmount", borderColor: "gray.200", placeholder: "0", type: "number", fontSize: "2xl", fontWeight: "semibold", _placeholder: { color: "gray.200" }, step: 0.0000000001, min: 0 }, register("topAmount"))),
                        react_2.default.createElement(react_1.InputRightElement, { w: "auto", justifyContent: "end", paddingX: 1.5, rounded: "lg" }, connected && (react_2.default.createElement(react_1.Menu, null,
                            react_2.default.createElement(react_1.MenuButton, { isDisabled: !connected, as: react_1.Button, leftIcon: react_2.default.createElement(react_1.Center, { w: 8, h: 8, color: "white", bg: "indigo.500", rounded: "full" }, base.icon), borderRadius: "20px 6px 6px 20px", paddingX: 1.5, bgColor: "gray.200", _hover: { cursor: "default" } }, base.ticker)))))),
                react_2.default.createElement(react_1.HStack, { justify: "center", alignItems: "center", position: "relative", paddingY: 2 },
                    react_2.default.createElement(react_1.Divider, { color: "gray.200" }),
                    react_2.default.createElement(react_1.Flex, null,
                        !connected && (react_2.default.createElement(react_1.Button, { size: "xs", colorScheme: "gray", variant: "outline", onClick: handleConnectWallet }, "Connect Wallet")),
                        connected && (react_2.default.createElement(react_1.Button, { size: "xs", colorScheme: "indigo", variant: "ghost", onClick: handleUseMax },
                            "Use Max (",
                            ownedBase > spendCap ? spendCap : ownedBase,
                            " ",
                            base.ticker,
                            ")"))),
                    react_2.default.createElement(react_1.Divider, { color: "gray.200" }),
                    react_2.default.createElement(react_1.IconButton, { isDisabled: !connected || isBaseSol, "aria-label": "Flip Tokens", size: "sm", colorScheme: "gray", rounded: "full", position: "absolute", right: 2, onClick: handleFlipTokens, icon: react_2.default.createElement(react_1.Icon, { as: ri_1.RiArrowUpDownFill, w: 5, h: 5 }) })),
                react_2.default.createElement(react_1.Flex, { flexDir: "column" },
                    react_2.default.createElement(react_1.Text, { color: "gray.600", fontSize: "xs" }, "You Receive"),
                    react_2.default.createElement(react_1.InputGroup, { size: "lg" },
                        react_2.default.createElement(react_1.Input, Object.assign({ isInvalid: !!errors.bottomAmount, isReadOnly: true, isDisabled: !connected, id: "bottomAmount", borderColor: "gray.200", placeholder: "0", type: "number", fontSize: "2xl", fontWeight: "semibold", step: 0.0000000001, min: 0, _placeholder: { color: "gray.200" }, _hover: { cursor: "not-allowed" }, _focus: { outline: "none", borderColor: "gray.200" } }, register("bottomAmount"))),
                        react_2.default.createElement(react_1.InputRightElement, { w: "auto", justifyContent: "end", paddingX: 1.5, rounded: "lg" }, connected && (react_2.default.createElement(react_1.Menu, null,
                            react_2.default.createElement(react_1.MenuButton, { isDisabled: !connected, as: react_1.Button, leftIcon: react_2.default.createElement(react_1.Center, { w: 8, h: 8, color: "white", bg: "indigo.500", rounded: "full" }, target.icon), borderRadius: "20px 6px 6px 20px", paddingX: 1.5, bgColor: "gray.200", _hover: { cursor: "default" } }, target.ticker)))))),
                react_2.default.createElement(react_1.VStack, { spacing: 1, padding: 4, align: "stretch", color: "gray.400", borderColor: "gray.200", borderWidth: "1px", rounded: "lg", fontSize: "sm" },
                    react_2.default.createElement(react_1.Flex, { justify: "space-between", alignItems: "center" },
                        react_2.default.createElement(react_1.Text, null, "Rate"),
                        react_2.default.createElement(react_1.Text, null, rate !== "--"
                            ? `1 ${base.ticker} ≈ ${rate} ${target.ticker}`
                            : rate)),
                    react_2.default.createElement(react_1.Flex, { justify: "space-between", alignItems: "center" },
                        react_2.default.createElement(react_1.HStack, null,
                            react_2.default.createElement(react_1.Text, null, "Slippage"),
                            react_2.default.createElement(react_1.Tooltip, { isDisabled: !connected, placement: "top", label: "Your transaction will fail if the price changes unfavorably more than this percentage.", portalProps: { containerRef: formRef } },
                                react_2.default.createElement(react_1.Flex, null,
                                    react_2.default.createElement(react_1.Icon, { w: 5, h: 5, as: ri_1.RiInformationLine, _hover: { color: "indigo.500", cursor: "pointer" } })))),
                        react_2.default.createElement(react_1.InputGroup, { size: "sm", w: "60px" },
                            react_2.default.createElement(react_1.Input, Object.assign({ isInvalid: !!errors.slippage, isDisabled: !connected, id: "slippage", borderColor: "gray.200", textAlign: "right", rounded: "lg", placeholder: "0", type: "number", fontWeight: "semibold", step: 1, min: 1, max: 90, paddingRight: 5, paddingLeft: 1 }, register("slippage"))),
                            react_2.default.createElement(react_1.InputRightElement, { w: 4, justifyContent: "end", paddingRight: 1.5, rounded: "lg" },
                                react_2.default.createElement(react_1.Text, { margin: 0 }, "%")))),
                    react_2.default.createElement(react_1.Flex, { justify: "space-between", alignItems: "center" },
                        react_2.default.createElement(react_1.Text, null, "Estimated Fees"),
                        react_2.default.createElement(react_1.Flex, null, fee)),
                    react_2.default.createElement(react_1.Flex, { justify: "space-between", alignItems: "center" },
                        react_2.default.createElement(react_1.HStack, null,
                            react_2.default.createElement(react_1.Text, null,
                                isBuying ? base.ticker : target.ticker,
                                " Royalties"),
                            react_2.default.createElement(react_1.Tooltip, { isDisabled: !connected, placement: "top", label: `A purchase fee in ${base.ticker} that is split amongst stakers of ${target.ticker}`, portalProps: { containerRef: formRef } },
                                react_2.default.createElement(react_1.Flex, null,
                                    react_2.default.createElement(react_1.Icon, { w: 5, h: 5, as: ri_1.RiInformationLine, _hover: { color: "indigo.500", cursor: "pointer" } })))),
                        react_2.default.createElement(react_1.Flex, null,
                            humanReadablePercentage(isBuying
                                ? tokenBonding.buyBaseRoyaltyPercentage
                                : tokenBonding.sellBaseRoyaltyPercentage),
                            "%")),
                    react_2.default.createElement(react_1.Flex, { justify: "space-between", alignItems: "center" },
                        react_2.default.createElement(react_1.HStack, null,
                            react_2.default.createElement(react_1.Text, null,
                                isBuying ? target.ticker : base.ticker,
                                " Royalties"),
                            react_2.default.createElement(react_1.Tooltip, { isDisabled: !connected, placement: "top", label: `A percentage of every ${target.ticker} token minted goes to the person who has claimed this token`, portalProps: { containerRef: formRef } },
                                react_2.default.createElement(react_1.Flex, null,
                                    react_2.default.createElement(react_1.Icon, { w: 5, h: 5, as: ri_1.RiInformationLine, _hover: { color: "indigo.500", cursor: "pointer" } })))),
                        react_2.default.createElement(react_1.Flex, null,
                            humanReadablePercentage(isBuying
                                ? tokenBonding.buyTargetRoyaltyPercentage
                                : tokenBonding.sellTargetRoyaltyPercentage),
                            "%"))),
                react_2.default.createElement(react_1.Box, { position: "relative" },
                    react_2.default.createElement(react_1.ScaleFade, { initialScale: 0.9, in: !hasBaseAmount || moreThanSpendCap },
                        react_2.default.createElement(react_1.Center, { bgColor: "gray.500", rounded: "md", paddingY: 2, color: "white", w: "full", position: "absolute", top: -10, fontSize: "sm" },
                            moreThanSpendCap && (react_2.default.createElement(react_1.Text, null,
                                "Spend Cap is ",
                                spendCap,
                                " ",
                                base.ticker,
                                ". Please adjust amount")),
                            !moreThanSpendCap && (react_2.default.createElement(react_1.Text, null,
                                "Insufficent funds for this trade.",
                                " ",
                                react_2.default.createElement(react_1.Text, { as: "u" },
                                    react_2.default.createElement(react_1.Link, { color: "indigo.100", _hover: { color: "indigo.200" }, onClick: handleBuyBase }, `Buy more now.`)))))),
                    react_2.default.createElement(react_1.Button, { isDisabled: !connected || !hasBaseAmount || moreThanSpendCap, w: "full", colorScheme: "indigo", size: "lg", type: "submit", isLoading: awaitingApproval || isSubmitting, loadingText: awaitingApproval
                            ? "Awaiting Approval"
                            : action === "buy"
                                ? "Buying"
                                : "Selling" }, "Trade"))))));
};
exports.SwapForm = SwapForm;
//# sourceMappingURL=SwapForm.js.map